@@ -39,7 +39,10 @@
   #define PY_SSIZE_T_MIN INT_MIN
   #define PY_SSIZE_T_MAX INT_MAX
   #define PY_FORMAT_SIZE_T ""
+  #define PyInt_FromSsize_t(x) PyInt_FromLong((x))
 
+  #define lenfunc inquiry
+  #define ssizeargfunc intargfunc
   #define readbufferproc getreadbufferproc
   #define writebufferproc getwritebufferproc
   #define segcountproc getsegcountproc
@@ -50,6 +53,10 @@
   #define CONV_CODE_PY_SSIZE_T "n"
 #endif
 
+#ifndef Py_TYPE
+  #define Py_TYPE(o) (((PyObject*)(o))->ob_type)
+#endif
+
 /* FORMAT_CODE_PY_SSIZE_T is for Py_ssize_t: */
 #define FORMAT_CODE_PY_SSIZE_T "%" PY_FORMAT_SIZE_T "d"
 
